


/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/

#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"

float causize = 10;
float cauintens = 40;
int scene = 3;
// Listener class for frame updates
class RaytraceDemoListener : public ExampleFrameListener
{
protected:
	
	SceneNode* objectNode;
	unsigned long framecount;
	
public:
    RaytraceDemoListener(RenderWindow* window, Camera* maincam, SceneNode* objectNode)
        :ExampleFrameListener(window, maincam) 
				 
    {
		this->mMoveSpeed = 2.0;
		framecount = 0;
		this->objectNode = objectNode;
	  }
	bool processUnbufferedKeyInput(const FrameEvent& evt)
    {
		static float lastTime = 0.0;
		float thisTime = (float) Root::getSingleton().getTimer()->getMilliseconds() / 1000.0f;
		float elapsed = thisTime - lastTime;
		float move = elapsed * 0.25;
		float rotate = 80.0 * move;

		if (mInputDevice->isKeyDown(KC_UP))
        {
			objectNode->translate(0,0,move);
        }
		if (mInputDevice->isKeyDown(KC_MULTIPLY))
        {
			//objectNode->scale(1.1,1.1,1.1);
			objectNode->rotate(Vector3(0,0,1), Degree(rotate));
			mTimeUntilNextToggle = 1;
        }
		if (mInputDevice->isKeyDown(KC_DIVIDE))
        {
			//objectNode->scale(1.0/1.1,1.0/1.1,1.0/1.1);
			objectNode->rotate(Vector3(1,0,0), Degree(rotate));
			mTimeUntilNextToggle = 1;
        }
		if (mInputDevice->isKeyDown(KC_ADD))
        {
			objectNode->rotate(Vector3(0,1,0), Degree(rotate));
        }
		if (mInputDevice->isKeyDown(KC_HOME))
        {
			objectNode->translate(0,move,0);
        }
		if (mInputDevice->isKeyDown(KC_END))
        {
			objectNode->translate(0,-move,0);
        }
		if (mInputDevice->isKeyDown(KC_DOWN))
        {
			objectNode->translate(0,0,-move);
        }
		if (mInputDevice->isKeyDown(KC_RIGHT))
        {
			objectNode->translate(move,0,0);
        }
		if (mInputDevice->isKeyDown(KC_LEFT))
        {
			objectNode->translate(-move,0,0);
        }

		if (mInputDevice->isKeyDown(KC_A))
        {
            // Move camera left
            mTranslateVector.x = -mMoveScale;
        }

        if (mInputDevice->isKeyDown(KC_D))
        {
            // Move camera RIGHT
            mTranslateVector.x = mMoveScale;
        }

        /* Move camera forward by keypress. */
        if ( mInputDevice->isKeyDown(KC_W) )
        {
            mTranslateVector.z = -mMoveScale;
        }
		
        /* Move camera backward by keypress. */
        if (mInputDevice->isKeyDown(KC_S) )
        {
            mTranslateVector.z = mMoveScale;
        }

        if (mInputDevice->isKeyDown(KC_PGUP))
        {
            // Move camera up
            mTranslateVector.y = mMoveScale;
        }

        if (mInputDevice->isKeyDown(KC_PGDOWN))
        {
            // Move camera down
            mTranslateVector.y = -mMoveScale;
        }       

        if( mInputDevice->isKeyDown( KC_ESCAPE) )
        {            
            return false;
        }

		// see if switching is on, and you want to toggle 
        if (mInputTypeSwitchingOn && mInputDevice->isKeyDown(KC_M) && mTimeUntilNextToggle <= 0)
        {
			switchMouseMode();
            mTimeUntilNextToggle = 1;
        }

        if (mInputTypeSwitchingOn && mInputDevice->isKeyDown(KC_K) && mTimeUntilNextToggle <= 0)
        {
			// must be going from immediate keyboard to buffered keyboard
			switchKeyMode();
            mTimeUntilNextToggle = 1;
        }
        if (mInputDevice->isKeyDown(KC_F) && mTimeUntilNextToggle <= 0)
        {
            mStatsOn = !mStatsOn;
            showDebugOverlay(mStatsOn);

            mTimeUntilNextToggle = 1;
        }
        if (mInputDevice->isKeyDown(KC_T) && mTimeUntilNextToggle <= 0)
        {
            switch(mFiltering)
            {
            case TFO_BILINEAR:
                mFiltering = TFO_TRILINEAR;
                mAniso = 1;
                break;
            case TFO_TRILINEAR:
                mFiltering = TFO_ANISOTROPIC;
                mAniso = 8;
                break;
            case TFO_ANISOTROPIC:
                mFiltering = TFO_BILINEAR;
                mAniso = 1;
                break;
            default:
                break;
            }
            MaterialManager::getSingleton().setDefaultTextureFiltering(mFiltering);
            MaterialManager::getSingleton().setDefaultAnisotropy(mAniso);


            showDebugOverlay(mStatsOn);

            mTimeUntilNextToggle = 1;
        }

        if (mInputDevice->isKeyDown(KC_SYSRQ) && mTimeUntilNextToggle <= 0)
        {
			char tmp[20];
			sprintf(tmp, "screenshot_%d.png", ++mNumScreenShots);
            mWindow->writeContentsToFile(tmp);
            mTimeUntilNextToggle = 0.5;
			mWindow->setDebugText(String("Wrote ") + tmp);
        }
		
		if (mInputDevice->isKeyDown(KC_R) && mTimeUntilNextToggle <=0)
		{
			mSceneDetailIndex = (mSceneDetailIndex+1)%3 ;
			switch(mSceneDetailIndex) {
				case 0 : mCamera->setPolygonMode(PM_SOLID) ; break ;
				case 1 : mCamera->setPolygonMode(PM_WIREFRAME) ; break ;
				case 2 : mCamera->setPolygonMode(PM_POINTS) ; break ;
			}
			mTimeUntilNextToggle = 0.5;
		}

        static bool displayCameraDetails = false;
        if (mInputDevice->isKeyDown(KC_P) && mTimeUntilNextToggle <= 0)
        {
            displayCameraDetails = !displayCameraDetails;
            mTimeUntilNextToggle = 0.5;
            if (!displayCameraDetails)
                mWindow->setDebugText("");
        }
        if (displayCameraDetails)
        {
            // Print camera details
            mWindow->setDebugText("P: " + StringConverter::toString(mCamera->getDerivedPosition()) + " " + 
                "O: " + StringConverter::toString(mCamera->getDerivedOrientation()));
        }


		lastTime = thisTime;
        // Return true to continue rendering
        return true;
	}
    bool frameStarted(const FrameEvent& evt)
    {
		static bool first = true;
	

	//	lightNode->setPosition( 100.0 * Vector3(cos(-(float)framecount/500.0),2.0,sin(-(float)framecount/500.0)));
		
		
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;
      return result;     
    }

	

};


class FloorRenderTargetListener : public RenderTargetListener
{
public:
	Entity* scene;
	FloorRenderTargetListener(Entity* scene)
	{
		this->scene = scene;
	}
	void preRenderTargetUpdate(const RenderTargetEvent& evt)
    {
		scene->getSubEntity(0)->setVisible(false);
		scene->getSubEntity(17)->setVisible(false);
		scene->getSubEntity(35)->setVisible(false);
		scene->getSubEntity(52)->setVisible(false);
    }
    void postRenderTargetUpdate(const RenderTargetEvent& evt)
    {
       scene->getSubEntity(0)->setVisible(true);
		scene->getSubEntity(17)->setVisible(true);
		scene->getSubEntity(35)->setVisible(true);
		scene->getSubEntity(52)->setVisible(true);
    }

};

class CeilRenderTargetListener : public RenderTargetListener
{
public:
	Entity* scene;
	CeilRenderTargetListener(Entity* scene)
	{
		this->scene = scene;
	}
	void preRenderTargetUpdate(const RenderTargetEvent& evt)
    {
		scene->getSubEntity(1)->setVisible(false);
		scene->getSubEntity(18)->setVisible(false);
		scene->getSubEntity(36)->setVisible(false);
		scene->getSubEntity(53)->setVisible(false);
    }
    void postRenderTargetUpdate(const RenderTargetEvent& evt)
    {
       scene->getSubEntity(1)->setVisible(true);
		scene->getSubEntity(18)->setVisible(true);
		scene->getSubEntity(36)->setVisible(true);
		scene->getSubEntity(53)->setVisible(true);
    }

};

class ReflectionCameraFrameListener : public FrameListener
{
public:
	Camera* floorReflCam;
	Camera* ceilReflCam;
	Camera* mainCamera;
    ReflectionCameraFrameListener(Camera* floorcam, Camera* ceilCam, Camera* mainCam)
	{
		floorReflCam = floorcam;
		ceilReflCam = ceilCam;
		mainCamera = mainCam;
	}
	bool frameStarted(const FrameEvent& evt)
    {
		floorReflCam->setOrientation(mainCamera->getOrientation());
        floorReflCam->setPosition(mainCamera->getPosition());

	//	ceilReflCam->setOrientation(mainCamera->getOrientation());
    //    ceilReflCam->setPosition(mainCamera->getPosition());

		return true;
	}
};
/** Application class */
class RaytraceDemoApplication : public ExampleApplication
{

public:

protected:
	SceneNode* objectNode;

	void createPostproc()
	{
		/*CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/Glow", true);*/

		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GameTools/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GameTools/ToneMap", true);		

	}
    

	void createScene(void)
    {

		createPostproc();
		
		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		
		OgreIlluminationManager::getSingleton().setMaxJoinRadius(420);		

		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);

		mCamera->setPosition(0,1,0);		
		mCamera->setFOVy(Radian(Degree(80)));
		mCamera->setFarClipDistance(200);
		mCamera->setNearClipDistance(0.1);
		
		switch (scene)
		{
		case 1:
			createScene1();break;
		case 2:
			createScene2();break;
		case 3:
			createScene3();break;
		}
		OgreIlluminationManager::getSingleton().initTechniques();
		
		
   }

   void createScene1()
   {
	   
        mSceneMgr->setAmbientLight(ColourValue(0.1, 0.0537, 0.0));
		//mSceneMgr->setAmbientLight(ColourValue(0.0, 0.0, 0.0));
		 

		// Skybox
        //mSceneMgr->setSkyBox(true, "Examples/MorningSkyBox");

		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

      	Entity* object = mSceneMgr->createEntity("object", "chinaman.mesh");
		object->setMaterialName("GameTools/Diffuse/use");
		//object->setMaterialName("GameTools/Diffuse2");
		objectNode = rootNode->createChildSceneNode();
		objectNode->attachObject(object);
	//	objectNode->scale(0.05,0.05,0.05);
		objectNode->scale(3,3,3);
	//	objectNode->setPosition(0,4,0);			
		objectNode->_updateBounds();

		Entity* room = mSceneMgr->createEntity("atlascube", "diffscene.mesh");
		//room->setMaterialName("GameTools/Phong");
		SceneNode* roomNode = rootNode->createChildSceneNode();
		//roomNode->setScale(10,10,10);
		roomNode->attachObject(room);
		roomNode->_updateBounds();
		
		Light* bluelight = mSceneMgr->createLight("BlueLight");
		bluelight->setType(Light::LT_POINT);
        bluelight->setDiffuseColour(ColourValue(0.152,0.824, 1.0));
		//bluelight->setSpecularColour(ColourValue(0.152,0.824, 1.0));
		bluelight->setPowerScale(5.726);
		bluelight->setAttenuation(12, 0, 1, 0);
		SceneNode* bluelightNode = rootNode->createChildSceneNode();
		bluelightNode->setPosition(-5.948,9.619,0.980);
		bluelightNode->attachObject(bluelight); 

		Light* walllight1 = mSceneMgr->createLight("WallLight1");
		walllight1->setType(Light::LT_POINT);
        walllight1->setDiffuseColour(ColourValue(0.988,1.0, 0.808));
		//walllight1->setSpecularColour(ColourValue(0.988,1.0, 0.808));
		walllight1->setPowerScale(2.5);
		walllight1->setAttenuation(12, 0, 1, 0);
		SceneNode* walllight1Node = rootNode->createChildSceneNode();
		walllight1Node->setPosition(-4.822,9.353,-5.194);
		walllight1Node->attachObject(walllight1); 

		Light* walllight2 = mSceneMgr->createLight("WallLight2");
		walllight2->setType(Light::LT_POINT);
        walllight2->setDiffuseColour(ColourValue(0.988,1.0, 0.808));
		//walllight2->setSpecularColour(ColourValue(0.988,1.0, 0.808));
		walllight2->setPowerScale(2.5);
		walllight2->setAttenuation(12, 0, 1, 0);
		SceneNode* walllight2Node = rootNode->createChildSceneNode();
		walllight2Node->setPosition(-4.822,9.353,5.150);
		walllight2Node->attachObject(walllight2); 

		Light* walllight3 = mSceneMgr->createLight("WallLight3");
		walllight3->setType(Light::LT_POINT);
        walllight3->setDiffuseColour(ColourValue(0.988,1.0, 0.808));
		//walllight3->setSpecularColour(ColourValue(0.988,1.0, 0.808));
		walllight3->setPowerScale(2.5);
		walllight3->setAttenuation(12, 0, 1, 0);
		SceneNode* walllight3Node = rootNode->createChildSceneNode();
		walllight3Node->setPosition(-11.08,10.07,-24.658);
		walllight3Node->attachObject(walllight3); 

		Light* walllight4 = mSceneMgr->createLight("WallLight4");
		walllight4->setType(Light::LT_POINT);
        walllight4->setDiffuseColour(ColourValue(0.988,1.0, 0.808));
		//walllight4->setSpecularColour(ColourValue(0.988,1.0, 0.808));
		walllight4->setPowerScale(2.5);
		walllight4->setAttenuation(12, 0, 1, 0);
		SceneNode* walllight4Node = rootNode->createChildSceneNode();
		walllight4Node->setPosition(-11.08,10.07,-11.474);
		walllight4Node->attachObject(walllight4); 

		Light* redlight1 = mSceneMgr->createLight("RedLight1");
		redlight1->setType(Light::LT_POINT);
        redlight1->setDiffuseColour(ColourValue(1.0,0.367, 0.016));
		//redlight1->setSpecularColour(ColourValue(1.0,0.367, 0.016));
		redlight1->setPowerScale(3.59);
		redlight1->setAttenuation(2, 0, 1, 0);
		SceneNode* redlight1Node = rootNode->createChildSceneNode();
		redlight1Node->setPosition(24.958,7.743,-10.838);
		redlight1Node->attachObject(redlight1); 

		Light* redlight2 = mSceneMgr->createLight("RedLight2");
		redlight2->setType(Light::LT_POINT);
        redlight2->setDiffuseColour(ColourValue(1.0,0.367, 0.016));
		//redlight2->setSpecularColour(ColourValue(1.0,0.367, 0.016));
		redlight2->setPowerScale(3.59);
		redlight2->setAttenuation(2, 0, 1, 0);
		SceneNode* redlight2Node = rootNode->createChildSceneNode();
		redlight2Node->setPosition(18.689,7.743,-10.838);
		redlight2Node->attachObject(redlight2); 
   }

   Camera* mReflectCamFloor;
   Camera* mReflectCamCeil;
   ReflectionCameraFrameListener* reflListener;
   void createScene2()
   {
	   mCamera->setPosition(0,5,10);
	   MovablePlane *floorPlane = new MovablePlane("ReflectPlaneFloor");
        floorPlane->d = 0;
        floorPlane->normal = Vector3::UNIT_Y;

		RenderTexture* rttTexFloor = mRoot->getRenderSystem()->createRenderTexture( "FloorReflection", 512, 512, TEX_TYPE_2D, PF_R8G8B8 );
        {
            mReflectCamFloor = mSceneMgr->createCamera("ReflectCamFloor");
            mReflectCamFloor->setNearClipDistance(mCamera->getNearClipDistance());
            mReflectCamFloor->setFarClipDistance(mCamera->getFarClipDistance());
           /* mReflectCamFloor->setAspectRatio(
                (Real)mWindow->getViewport(0)->getActualWidth() / 
                (Real)mWindow->getViewport(0)->getActualHeight());*/
			mReflectCamFloor->setFOVy(mCamera->getFOVy());
			mReflectCamFloor->setAspectRatio(mCamera->getAspectRatio());

            Viewport *v = rttTexFloor->addViewport( mReflectCamFloor );
            v->setClearEveryFrame( true );
            v->setBackgroundColour( ColourValue::Black );
			
          
            // set up linked reflection
            mReflectCamFloor->enableReflection(floorPlane);
            // Also clip
            mReflectCamFloor->enableCustomNearClipPlane(floorPlane);
        }
	/*	MovablePlane *ceilPlane = new MovablePlane("ReflectPlaneCeil");
        ceilPlane->d = -8;
		ceilPlane->normal = Vector3::NEGATIVE_UNIT_Y;
		RenderTexture* rttTexCeil = mRoot->getRenderSystem()->createRenderTexture( "CeilReflection", 512, 512, TEX_TYPE_2D, PF_R8G8B8 );
        {
            mReflectCamCeil = mSceneMgr->createCamera("ReflectCamCeil");
            mReflectCamCeil->setNearClipDistance(mCamera->getNearClipDistance());
            mReflectCamCeil->setFarClipDistance(mCamera->getFarClipDistance());
            mReflectCamCeil->setAspectRatio(
                (Real)mWindow->getViewport(0)->getActualWidth() / 
                (Real)mWindow->getViewport(0)->getActualHeight());

            Viewport *v = rttTexCeil->addViewport( mReflectCamCeil );
            v->setClearEveryFrame( true );
            v->setBackgroundColour( ColourValue::Black );
			
           
            // set up linked reflection
            mReflectCamCeil->enableReflection(floorPlane);
            // Also clip
            mReflectCamCeil->enableCustomNearClipPlane(floorPlane);
        }
		*/
		reflListener = new ReflectionCameraFrameListener(mReflectCamFloor, mReflectCamCeil, mCamera);
		reflListener->setPriority(2);

	   // mSceneMgr->setAmbientLight(ColourValue(0.1, 0.0537, 0.0));
	   mSceneMgr->setAmbientLight(ColourValue(0.2, 0.25, 0.3));
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

      	Entity* object = mSceneMgr->createEntity("object", "buddha.mesh");
		object->setMaterialName("GameTools/Diffuse/use");
		//object->setMaterialName("GameTools/Diffuse2");
		objectNode = rootNode->createChildSceneNode();
		objectNode->attachObject(object);
		objectNode->scale(1.8,1.8,1.8);
		objectNode->rotate(Vector3(0,1,0),Radian(Math::PI));
		objectNode->setPosition(0,0,0);
		objectNode->_updateBounds();

		Entity* room = mSceneMgr->createEntity("scene", "uvegfolyoso2.mesh");
		//room->setMaterialName("GameTools/Phong");
		SceneNode* roomNode = rootNode->createChildSceneNode();
		roomNode->attachObject(room);
		roomNode->_updateBounds();
		rttTexFloor->addListener(new FloorRenderTargetListener(room));
		//rttTexCeil->addListener(new CeilRenderTargetListener(room));

		Light* light1 = mSceneMgr->createLight("Light1");
		light1->setType(Light::LT_POINT);
        light1->setDiffuseColour(ColourValue(0.526,0.535, 0.974));
		light1->setPowerScale(3);
		SceneNode* lightNode1 = rootNode->createChildSceneNode();
		lightNode1->setPosition(-1.266,4.131,-4.438);
		lightNode1->attachObject(light1); 

		Light* light2 = mSceneMgr->createLight("Light2");
		light2->setType(Light::LT_POINT);
        light2->setDiffuseColour(ColourValue(0.391,0.781, 0.974));
		light2->setPowerScale(3);
		SceneNode* lightNode2 = rootNode->createChildSceneNode();
		lightNode2->setPosition(-33.537,4.131,-4.438);
		lightNode2->attachObject(light2); 

		Light* light3 = mSceneMgr->createLight("Light3");
		light3->setType(Light::LT_POINT);
        light3->setDiffuseColour(ColourValue(0.391,0.781, 0.974));
		light3->setPowerScale(3);
		SceneNode* lightNode3 = rootNode->createChildSceneNode();
		lightNode3->setPosition(-97.446,4.131,-4.438);
		lightNode3->attachObject(light3); 

		Light* light4 = mSceneMgr->createLight("Light4");
		light4->setType(Light::LT_POINT);
        light4->setDiffuseColour(ColourValue(0.304,0.974, 0.770));
		light4->setPowerScale(3);
		SceneNode* lightNode4 = rootNode->createChildSceneNode();
		lightNode4->setPosition(-63.277,4.131,-4.438);
		lightNode4->attachObject(light4); 

		



   }

   void createScene3()
   {
	   OgreIlluminationManager::getSingleton().setFocusingSM(false);
	   OgreIlluminationManager::getSingleton().setShadowMapSize(512);
	   OgreIlluminationManager::getSingleton().setBlurShadowMap(true);
	   OgreIlluminationManager::getSingleton().setShadowMapMaterialName("GameTools/ShadowMapDistance");
	    mSceneMgr->setAmbientLight(ColourValue(0.505 * 0.2, 0.897 * 0.2, 0.914 * 0.2));
		
		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

     	Entity* object = mSceneMgr->createEntity("object", "buddha.mesh");
		object->setMaterialName("GameTools/Diffuse/use");
		//object->setMaterialName("GameTools/DiffuseP2P");
		objectNode = rootNode->createChildSceneNode();
		objectNode->attachObject(object);
		objectNode->scale(0.4,0.4,0.4);
		objectNode->rotate(Vector3(0,1,0),Radian(Math::PI));
		objectNode->setPosition(0,0,0);
		objectNode->_updateBounds();

		Entity* room = mSceneMgr->createEntity("scene", "difflab.mesh");
		//room->setMaterialName("GameTools/Phong");
		SceneNode* roomNode = rootNode->createChildSceneNode();
		roomNode->attachObject(room);
		roomNode->_updateBounds();		

		mSceneMgr->setShadowTechnique(SHADOWTYPE_NONE);
		Light* redLight1 = mSceneMgr->createLight("RedLight1");
		redLight1->setType(Light::LT_POINT);
		redLight1->setCastShadows(false);
        redLight1->setDiffuseColour(ColourValue(1,0.055, 0.0));
		redLight1->setPowerScale(1.6);
		SceneNode* redLightNode1 = rootNode->createChildSceneNode();
		redLightNode1->setPosition(0.05,2.361,-1.07);
		redLightNode1->attachObject(redLight1);

		Light* greenLight1 = mSceneMgr->createLight("GreenLight1");
		greenLight1->setType(Light::LT_POINT);
		greenLight1->setCastShadows(false);
        greenLight1->setDiffuseColour(ColourValue(0.362,0.783, 0.685));
		greenLight1->setPowerScale(2.0);
		SceneNode* greenLightNode1 = rootNode->createChildSceneNode();
		greenLightNode1->setPosition(1.312,1.313,0);
		greenLightNode1->attachObject(greenLight1); 

		Light* mainBlueLight = mSceneMgr->createLight("MainBlueLight");
		mainBlueLight->setType(Light::LT_SPOTLIGHT);
		mainBlueLight->setCastShadows(true);
        mainBlueLight->setDiffuseColour(ColourValue(0.395,0.766, 1.0));
		mainBlueLight->setPowerScale(5.0);
		mainBlueLight->setSpotlightRange(Degree(0),Degree(160));
		mainBlueLight->setAttenuation(2.5, 0, 1, 0);
		SceneNode* mainBlueLightNode = rootNode->createChildSceneNode();
		mainBlueLightNode->setPosition(-0.546,2.095,1.035);
		//mainBlueLightNode->rotate(Vector3::UNIT_X, Degree(161.396));
		//mainBlueLightNode->rotate(Vector3::UNIT_Y, Degree(53.955));
		//mainBlueLightNode->rotate(Vector3::UNIT_Z, Degree(-145.065));
		mainBlueLight->setDirection(-1.5,-2.0,1.0);
		mainBlueLightNode->attachObject(mainBlueLight); 
/*
		Light* blueLight1 = mSceneMgr->createLight("BlueLight1");
		blueLight1->setType(Light::LT_SPOTLIGHT);
		blueLight1->setCastShadows(false);
        blueLight1->setDiffuseColour(ColourValue(0.395,0.766, 1.0));
		blueLight1->setPowerScale(0.7);
		blueLight1->setSpotlightRange(Degree(20),Degree(90));
		SceneNode* blueLightNode = rootNode->createChildSceneNode();
		blueLightNode->setPosition(-0.9,2.095,0.185);
		//blueLightNode->rotate(Vector3::UNIT_X, Degree(148.785));
		//blueLightNode->rotate(Vector3::UNIT_Y, Degree(39.001));
		//blueLightNode->rotate(Vector3::UNIT_Z, Degree(-154.304));
		blueLight1->setDirection(-1.5,-2.0,1.0);
		blueLightNode->attachObject(blueLight1); 

		Light* blueLight2 = mSceneMgr->createLight("BlueLight2");
		blueLight2->setType(Light::LT_SPOTLIGHT);
		blueLight2->setCastShadows(false);
        blueLight2->setDiffuseColour(ColourValue(0.395,0.766, 1.0));
		blueLight2->setPowerScale(0.7);
		blueLight2->setSpotlightRange(Degree(20),Degree(90));
		SceneNode* blueLightNode2 = rootNode->createChildSceneNode();
		blueLightNode2->setPosition(0.091,2.095,1.556);
		//blueLightNode2->rotate(Vector3::UNIT_X, Degree(-112.941));
		//blueLightNode2->rotate(Vector3::UNIT_Y, Degree(68.432));
		//blueLightNode2->rotate(Vector3::UNIT_Z, Degree(-66.441));
		blueLight2->setDirection(-1.5,-2.0,1.0);
		blueLightNode2->attachObject(blueLight2); */
   }

   void createPlane(Ogre::String _name, Ogre::String _material, Ogre::Vector3 _pos, Ogre::Vector2 _size, Ogre::Vector2 _subdivisions = Ogre::Vector2(1,1)) {
		
	   Plane _plane;
        _plane.normal = Vector3::UNIT_Y;
        _plane.d = 0;


        MeshManager::getSingleton().createPlane(_name + ".plane",
            ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
            _plane,_size.x,_size.y,1,1,true,1,_subdivisions.x,_subdivisions.y,Vector3::UNIT_Z);
        
		Entity *_entity = mSceneMgr->createEntity(_name + ".entity", _name + ".plane");
        _entity->setMaterialName(_material);

		SceneNode* _node = mSceneMgr->getRootSceneNode()->createChildSceneNode(_name + ".node");
		_node->attachObject(_entity);
		_node->setPosition(_pos);
	}



	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new RaytraceDemoListener(mWindow,
											mCamera,
											objectNode
											);
		mFrameListener->setPriority(1);
		OgreIlluminationManager::getSingleton().setPriority(10);
        mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
		mRoot->addFrameListener(mFrameListener);
		if(scene == 2)
		mRoot->addFrameListener(reflListener);
	
    }

};


